@@ -34,8 +34,7 @@
 #include "gnome-vfs-monitor.h"
 #include "gnome-vfs-ops.h"
 #include "gnome-vfs-utils.h"
-#include "eggdesktopentries.h"
-#include "eggdirfuncs.h"
+#include "xdgmime.h"
 
 typedef struct {
 	char *path;
@@ -56,6 +55,7 @@ typedef struct {
 } GnomeVFSMimeInfoCache;
 
 extern void _gnome_vfs_mime_monitor_emit_data_changed (GnomeVFSMIMEMonitor *monitor); 
+extern void _gnome_vfs_mime_info_cache_init (void);
 
 static void gnome_vfs_mime_info_cache_dir_init (GnomeVFSMimeInfoCacheDir *dir);
 static void gnome_vfs_mime_info_cache_dir_init_defaults_list (GnomeVFSMimeInfoCacheDir *dir);
@@ -68,7 +68,6 @@ static gboolean gnome_vfs_mime_info_cache_dir_desktop_entry_is_valid (GnomeVFSMi
 static void gnome_vfs_mime_info_cache_dir_add_desktop_entries (GnomeVFSMimeInfoCacheDir *dir,
 							       const char *mime_type,
 							       char **new_desktop_file_ids);
-static void gnome_vfs_mime_info_cache_init (void);
 static GnomeVFSMimeInfoCache *gnome_vfs_mime_info_cache_new (void);
 static void gnome_vfs_mime_info_cache_free (GnomeVFSMimeInfoCache *cache);
 
@@ -130,16 +129,14 @@ gnome_vfs_mime_info_cache_blow_global_cache (void)
 static void
 gnome_vfs_mime_info_cache_dir_init (GnomeVFSMimeInfoCacheDir *dir)
 {
-	EggDesktopEntries *entries;
 	GError *load_error;
+	GKeyFile *key_file;
 	gchar *filename, **mime_types;
 	int i;
-	static gchar *allowed_start_groups[] = { "MIME Cache", NULL };
 	struct stat buf;
 
 	load_error = NULL;
 	mime_types = NULL;
-	entries = NULL;
 
 	if (dir->mime_info_cache_map != NULL &&
 	    dir->cache_monitor_handle == NULL &&
@@ -155,6 +152,8 @@ gnome_vfs_mime_info_cache_dir_init (GnomeVFSMimeInfoCacheDir *dir)
 							  (GDestroyNotify) g_free,
 							  NULL);
 
+	key_file = g_key_file_new ();
+
 	filename = g_build_filename (dir->path, "mimeinfo.cache", NULL);
 
 	if (stat (filename, &buf) < 0)
@@ -165,32 +164,27 @@ gnome_vfs_mime_info_cache_dir_init (GnomeVFSMimeInfoCacheDir *dir)
 
 	dir->mime_info_cache_timestamp = buf.st_mtime;
 
-	entries =
-		egg_desktop_entries_new_from_file (allowed_start_groups,
-						   EGG_DESKTOP_ENTRIES_GENERATE_LOOKUP_MAP |
-						   EGG_DESKTOP_ENTRIES_DISCARD_COMMENTS |
-						   EGG_DESKTOP_ENTRIES_DISCARD_TRANSLATIONS,
-						   filename,
-						   &load_error);
+	g_key_file_load_from_file (key_file, filename, G_KEY_FILE_NONE, &load_error);
+
 	g_free (filename);
 	filename = NULL;
 
 	if (load_error != NULL)
 		goto error;
 
-	mime_types = egg_desktop_entries_get_keys (entries, "MIME Cache",
-						   NULL, &load_error);
+	mime_types = g_key_file_get_keys (key_file, "MIME Cache",
+					  NULL, &load_error);
 
 	if (load_error != NULL)
 		goto error;
 
 	for (i = 0; mime_types[i] != NULL; i++) {
 		gchar **desktop_file_ids;
-		desktop_file_ids = egg_desktop_entries_get_string_list (entries,
-									"MIME Cache",
-									mime_types[i],
-									NULL,
-									&load_error);
+		desktop_file_ids = g_key_file_get_string_list (key_file,
+							       "MIME Cache",
+							       mime_types[i],
+							       NULL,
+							       &load_error);
 
 		if (load_error != NULL) {
 			g_error_free (load_error);
@@ -206,15 +200,14 @@ gnome_vfs_mime_info_cache_dir_init (GnomeVFSMimeInfoCacheDir *dir)
 	}
 
 	g_strfreev (mime_types);
-	egg_desktop_entries_free (entries);
+	g_key_file_free (key_file);
 
 	return;
 error:
 	if (filename)
 		g_free (filename);
 
-	if (entries != NULL)
-		egg_desktop_entries_free (entries);
+	g_key_file_free (key_file);
 
 	if (mime_types != NULL)
 		g_strfreev (mime_types);
@@ -226,17 +219,15 @@ error:
 static void
 gnome_vfs_mime_info_cache_dir_init_defaults_list (GnomeVFSMimeInfoCacheDir *dir)
 {
-	EggDesktopEntries *entries;
+	GKeyFile *key_file;
 	GError *load_error;
 	gchar *filename, **mime_types;
 	char **desktop_file_ids;
 	int i;
 	struct stat buf;
-	static gchar *allowed_start_groups[] = { "Default Applications", NULL };
 
 	load_error = NULL;
 	mime_types = NULL;
-	entries = NULL;
 
 	if (dir->defaults_list_map != NULL &&
 	    dir->defaults_monitor_handle == NULL &&
@@ -251,6 +242,8 @@ gnome_vfs_mime_info_cache_dir_init_defaults_list (GnomeVFSMimeInfoCacheDir *dir)
 	dir->defaults_list_map = g_hash_table_new_full (g_str_hash, g_str_equal,
 							g_free, (GDestroyNotify)g_strfreev);
 
+	key_file = g_key_file_new ();
+
 	filename = g_build_filename (dir->path, "defaults.list", NULL);
 
 	if (stat (filename, &buf) < 0)
@@ -261,52 +254,46 @@ gnome_vfs_mime_info_cache_dir_init_defaults_list (GnomeVFSMimeInfoCacheDir *dir)
 
 	dir->defaults_list_timestamp = buf.st_mtime;
 
-	entries =
-		egg_desktop_entries_new_from_file (allowed_start_groups,
-						   EGG_DESKTOP_ENTRIES_GENERATE_LOOKUP_MAP |
-						   EGG_DESKTOP_ENTRIES_DISCARD_COMMENTS |
-						   EGG_DESKTOP_ENTRIES_DISCARD_TRANSLATIONS,
-						   filename,
-						   &load_error);
+	g_key_file_load_from_file (key_file, filename, G_KEY_FILE_NONE, &load_error);
+
 	g_free (filename);
 	filename = NULL;
 
 	if (load_error != NULL)
 		goto error;
 
-	mime_types = egg_desktop_entries_get_keys (entries, "Default Applications",
-						   NULL, &load_error);
+	mime_types = g_key_file_get_keys (key_file, "Default Applications",
+					  NULL, &load_error);
 
 	if (load_error != NULL)
 		goto error;
 
 	for (i = 0; mime_types[i] != NULL; i++) {
-		desktop_file_ids = egg_desktop_entries_get_string_list (entries,
-								       "Default Applications",
-								       mime_types[i],
-								       NULL,
-								       &load_error);
+		desktop_file_ids = g_key_file_get_string_list (key_file,
+							       "Default Applications",
+							       mime_types[i],
+							       NULL,
+							       &load_error);
 		if (load_error != NULL) {
 			g_error_free (load_error);
 			load_error = NULL;
 			continue;
 		}
-		
+
 		g_hash_table_replace (dir->defaults_list_map,
 				      g_strdup (mime_types[i]),
 				      desktop_file_ids);
 	}
 
 	g_strfreev (mime_types);
-	egg_desktop_entries_free (entries);
+	g_key_file_free (key_file);
 
 	return;
 error:
 	if (filename)
 		g_free (filename);
 
-	if (entries != NULL)
-		egg_desktop_entries_free (entries);
+	g_key_file_free (key_file);
 
 	if (mime_types != NULL)
 		g_strfreev (mime_types);
@@ -315,6 +302,13 @@ error:
 		g_error_free (load_error);
 }
 
+static gboolean
+emit_mime_changed (gpointer data)
+{
+	_gnome_vfs_mime_monitor_emit_data_changed (gnome_vfs_mime_monitor_get ());
+	return FALSE;
+}
+
 static void
 gnome_vfs_mime_info_cache_dir_changed (GnomeVFSMonitorHandle    *handle,
 		                       const gchar              *monitor_uri,
@@ -325,7 +319,9 @@ gnome_vfs_mime_info_cache_dir_changed (GnomeVFSMonitorHandle    *handle,
 	G_LOCK (mime_info_cache);
 	gnome_vfs_mime_info_cache_blow_global_cache ();
 	gnome_vfs_mime_info_cache_dir_init (dir);
+	mime_info_cache->should_ping_mime_monitor = FALSE;
 	G_UNLOCK (mime_info_cache);
+	g_idle_add (emit_mime_changed, NULL);
 }
 
 static void
@@ -338,7 +334,9 @@ gnome_vfs_mime_info_cache_dir_defaults_changed (GnomeVFSMonitorHandle    *handle
 	G_LOCK (mime_info_cache);
 	gnome_vfs_mime_info_cache_blow_global_cache ();
 	gnome_vfs_mime_info_cache_dir_init_defaults_list (dir);
+	mime_info_cache->should_ping_mime_monitor = FALSE;
 	G_UNLOCK (mime_info_cache);
+	g_idle_add (emit_mime_changed, NULL);
 }
 
 static GnomeVFSMimeInfoCacheDir *
@@ -406,20 +404,19 @@ static char **
 gnome_vfs_mime_info_cache_get_search_path (void)
 {
 	char **args = NULL;
-	char **data_dirs;
-	char *user_data_dir;
+	const char * const *data_dirs;
+	const char *user_data_dir;
 	int i, length, j;
 
-	data_dirs = egg_get_secondary_data_dirs ();
+	data_dirs = g_get_system_data_dirs ();
 
 	for (length = 0; data_dirs[length] != NULL; length++);
 
 	args = g_new (char *, length + 2);
 
 	j = 0;
-	user_data_dir = egg_get_user_data_dir ();
+	user_data_dir = g_get_user_data_dir ();
 	args[j++] = g_build_filename (user_data_dir, "applications", NULL);
-	g_free (user_data_dir);
 
 	for (i = 0; i < length; i++) {
 		args[j++] = g_build_filename (data_dirs[i],
@@ -427,8 +424,6 @@ gnome_vfs_mime_info_cache_get_search_path (void)
 	}
 	args[j++] = NULL;
 
-	g_strfreev (data_dirs);
-
 	return args;
 }
 
@@ -436,7 +431,7 @@ static gboolean
 gnome_vfs_mime_info_cache_dir_desktop_entry_is_valid (GnomeVFSMimeInfoCacheDir *dir,
                                                       const char *desktop_entry)
 {
-	EggDesktopEntries *entries;
+	GKeyFile *key_file;
 	GError *load_error;
 	int i;
 	gboolean can_show_in;
@@ -446,12 +441,10 @@ gnome_vfs_mime_info_cache_dir_desktop_entry_is_valid (GnomeVFSMimeInfoCacheDir *
 	can_show_in = TRUE;
 
 	filename = g_build_filename ("applications", desktop_entry, NULL);
-	entries =
-		egg_desktop_entries_new_from_file (NULL,
-						   EGG_DESKTOP_ENTRIES_DISCARD_COMMENTS |
-						   EGG_DESKTOP_ENTRIES_DISCARD_TRANSLATIONS,
-						   filename,
-						   &load_error);
+
+	key_file = g_key_file_new ();
+	g_key_file_load_from_data_dirs (key_file, filename, NULL,
+					G_KEY_FILE_NONE, &load_error);
 	g_free (filename);
 
 	if (load_error != NULL) {
@@ -459,21 +452,20 @@ gnome_vfs_mime_info_cache_dir_desktop_entry_is_valid (GnomeVFSMimeInfoCacheDir *
 		return FALSE;
 	}
 
-	if (egg_desktop_entries_has_key (entries,
-					 egg_desktop_entries_get_start_group (entries),
-					 "OnlyShowIn")) {
+	if (g_key_file_has_key (key_file, DESKTOP_ENTRY_GROUP,
+				"OnlyShowIn", NULL)) {
 
 		char **only_show_in_list;
-		only_show_in_list = egg_desktop_entries_get_string_list (entries,
-									 egg_desktop_entries_get_start_group (entries),
-									 "OnlyShowIn",
-									 NULL,
-									 &load_error);
+		only_show_in_list = g_key_file_get_string_list (key_file,
+								DESKTOP_ENTRY_GROUP,
+								"OnlyShowIn",
+								NULL,
+								&load_error);
 
 		if (load_error != NULL) {
 			g_error_free (load_error);
 			g_strfreev (only_show_in_list);
-			egg_desktop_entries_free (entries);
+			g_key_file_free (key_file);
 			return FALSE;
 		}
 
@@ -488,15 +480,15 @@ gnome_vfs_mime_info_cache_dir_desktop_entry_is_valid (GnomeVFSMimeInfoCacheDir *
 		g_strfreev (only_show_in_list);
 	}
 
-	if (egg_desktop_entries_has_key (entries,
-					 egg_desktop_entries_get_start_group (entries),
-					 "NotShowIn")) {
+	if (g_key_file_has_key (key_file,
+				DESKTOP_ENTRY_GROUP,
+				"NotShowIn", NULL)) {
 		char **not_show_in_list;
-		not_show_in_list = egg_desktop_entries_get_string_list (entries,
-									egg_desktop_entries_get_start_group (entries),
-									"NotShowIn",
-									NULL,
-									&load_error);
+		not_show_in_list = g_key_file_get_string_list (key_file,
+							       DESKTOP_ENTRY_GROUP,
+							       "NotShowIn",
+							       NULL,
+							       &load_error);
 
 		if (load_error != NULL) {
 			g_error_free (load_error);
@@ -513,7 +505,7 @@ gnome_vfs_mime_info_cache_dir_desktop_entry_is_valid (GnomeVFSMimeInfoCacheDir *
 		g_strfreev (not_show_in_list);
 	}
 
-	egg_desktop_entries_free (entries);
+	g_key_file_free (key_file);
 	return can_show_in;
 }
 
@@ -605,16 +597,8 @@ gnome_vfs_mime_info_cache_update_dir_lists (void)
 	}
 }
 
-static gboolean
-emit_mime_changed (gpointer data)
-{
-	_gnome_vfs_mime_monitor_emit_data_changed (gnome_vfs_mime_monitor_get ());
-	return FALSE;
-}
-
-
-static void
-gnome_vfs_mime_info_cache_init (void)
+void
+_gnome_vfs_mime_info_cache_init (void)
 {
 	G_LOCK (mime_info_cache);
 	if (mime_info_cache == NULL) {
@@ -679,55 +663,24 @@ gnome_vfs_mime_info_cache_reload (const char *dir)
 	}
 }
 
-
-
-
-static void
-get_all_parent_types_helper (GList **mime_types, const char *mime_type)
-{
-	const gchar *parent_list;
-	
-	if (!g_list_find_custom (*mime_types, mime_type,
-				 (GCompareFunc) strcmp)) {
-		*mime_types = g_list_prepend (*mime_types, g_strdup (mime_type));
-	}
-
-	parent_list = gnome_vfs_mime_get_value (mime_type, "parent_classes");
-	if (parent_list) {
-		char **parents;
-		int i;
-		
-		parents = g_strsplit (parent_list, ":", -1);
-		for (i = 0; parents && parents[i] != NULL; i++) {
-			get_all_parent_types_helper (mime_types, parents[i]);
-		}
-		g_strfreev (parents);
-	}
-
-	if (g_str_has_prefix (mime_type, "text/")) {
-		if (!g_list_find_custom (*mime_types, "text/plain",
-					 (GCompareFunc) strcmp)) {
-			*mime_types = g_list_prepend (*mime_types, g_strdup ("text/plain"));
-		}
-	}
-	if (!g_str_has_prefix (mime_type, "inode/")) {
-		if (!g_list_find_custom (*mime_types, "application/octet-stream",
-					 (GCompareFunc) strcmp)) {
-			*mime_types = g_list_prepend (*mime_types, g_strdup ("application/octet-stream"));
-		}
-	}
-}
-
 static GList *
 get_all_parent_types (const char *mime_type)
 {
-	GList *mime_types;
+	const char *umime;
+	const char **parents;
+	GList *l = NULL;
+	int i;
 
-	/* TODO: Unalias mime_type first */
-	
-	mime_types = NULL;
-	get_all_parent_types_helper (&mime_types, mime_type);
-	return g_list_reverse (mime_types);
+	umime = xdg_mime_unalias_mime_type (mime_type);
+	l = g_list_prepend (l, g_strdup (umime));
+
+	parents = xdg_mime_get_mime_parents (mime_type);
+
+	for (i = 0; parents && parents[i] != NULL; i++) {
+		l = g_list_prepend (l, g_strdup (parents[i]));
+	}
+
+	return g_list_reverse (l);
 }
 
 GList *
@@ -738,7 +691,7 @@ gnome_vfs_mime_get_all_desktop_entries (const char *mime_type)
 	GnomeVFSMimeInfoCacheDir *dir;
 	char *type;
 
-	gnome_vfs_mime_info_cache_init ();
+	_gnome_vfs_mime_info_cache_init ();
 
 	G_LOCK (mime_info_cache);
 	mime_types = get_all_parent_types (mime_type);
@@ -777,8 +730,8 @@ gnome_vfs_mime_get_all_desktop_entries (const char *mime_type)
 	return desktop_entries;
 }
 
-gchar *
-gnome_vfs_mime_get_default_desktop_entry (const char *mime_type)
+static gchar *
+get_default_desktop_entry (const char *mime_type)
 {
 	gchar *desktop_entry;
 	char **desktop_entries;
@@ -786,14 +739,9 @@ gnome_vfs_mime_get_default_desktop_entry (const char *mime_type)
 	GnomeVFSMimeInfoCacheDir *dir;
 	int i;
 
-	gnome_vfs_mime_info_cache_init ();
-
-	G_LOCK (mime_info_cache);
-
 	desktop_entry = g_hash_table_lookup (mime_info_cache->global_defaults_cache,
 					     mime_type);
 	if (desktop_entry) {
-		G_UNLOCK (mime_info_cache);
 		return g_strdup (desktop_entry);
 	}
 
@@ -808,13 +756,38 @@ gnome_vfs_mime_get_default_desktop_entry (const char *mime_type)
 			    gnome_vfs_mime_info_desktop_entry_is_valid (desktop_entry)) {
 				g_hash_table_insert (mime_info_cache->global_defaults_cache,
 						     g_strdup (mime_type), g_strdup (desktop_entry));
-				G_UNLOCK (mime_info_cache);
 				return g_strdup (desktop_entry);
 			}
 		}
 	}
-	G_UNLOCK (mime_info_cache);
 
 	return NULL;
 }
 
+gchar *
+gnome_vfs_mime_get_default_desktop_entry (const char *mime_type)
+{
+	char *desktop_entry = NULL;
+	GList *mime_types, *m_list;
+
+	_gnome_vfs_mime_info_cache_init ();
+
+	G_LOCK (mime_info_cache);
+
+	mime_types = get_all_parent_types (mime_type);
+
+	for (m_list = mime_types; m_list != NULL; m_list = m_list->next) {
+		desktop_entry = get_default_desktop_entry (m_list->data);
+		if (desktop_entry) {
+			break;
+		}		
+	}
+
+	G_UNLOCK (mime_info_cache);
+
+	g_list_foreach (mime_types, (GFunc)g_free, NULL);
+	g_list_free (mime_types);
+
+	return desktop_entry;
+}
+
